/*
SST.js: High-Contrast, Small-Scale Structures
================================================================================

Kontrastverbesserung für kleine Strukturen

SST implementiert den Abschnitt "High-Contrast, Small-Scale Structures" aus dem
Tutorial "Dynamic Range and Local Contrast" von Vicent Peris
(https://pixinsight.com/tutorials/NGC7023-HDR/#First_Dynamic_Range_Compression)
Benötigt wird ein vorhandenes RAW-Bild, eine Kopie des fertigen RGB in der
linearen Form. Im Dialog wird das RAW-Bild ausgewählt. Zuerst wird das aktive,
nichtlineare Bild mit HDRWaveletTransform in den hellen Bereichen komprimiert
und danach SST.js aufgerufen. Die Anzahl der Iterationen bestimmt den Effekt.

SST implements the Section "High-Contrast, Small-Scale Structures" in
Vicent Peris' tutorial
https://pixinsight.com/tutorials/NGC7023-HDR/#First_Dynamic_Range_Compression
The script needs an existing RAW image, a copy of the linear RGB before
delinearization. This RAW image is referenced in the dialog window. Before
running SST.js, the active, nonlinear image must be compressed with the
HDRWaveletTransform process. The effect of SST is controlled by the number
of iterations.
================================================================================
 *
 * Input
 *
 * 1. Active window, HDR compressed
 *
 * 2. RAW image selected by a ViewList
 *       this is the linear RGB before stretching
 *
 * 3. optional export of the difference mask
 *
 * Copyright Hartmut V. Bornemann, 2016
 */

#feature-id    Utilities > SST

#include <pjsr/StdButton.jsh>
#include <pjsr/FrameStyle.jsh>
#include <pjsr/TextAlign.jsh>
#include <pjsr/NumericControl.jsh>
#include <pjsr/UndoFlag.jsh>
#include <pjsr/SampleType.jsh>
#include <pjsr/DataType.jsh>

#define TITLE "Transfer small scales to HDR compressed (L)RGB"
#define VERSION "1.0.0"
#define StdCursor_Wait 16
#define StdCursor_Arrow 1
#define StdCursor_ArrowQuestion 17


function sstDialog(hdrwtView)
{
   // Add all properties and methods of the core Dialog object to this object.
   this.__base__ = Dialog;
   this.__base__();

   var dlg = this;

   var labelWidth1 = this.font.width( "Contrast Limit:" );

   var rawView;

   var iterations = 1;
   var export_mask = false;

   // ------------------------------------------------------------------------
   // GUI
   // ------------------------------------------------------------------------

   this.lblHeadLine = new Label(this)
   with (this.lblHeadLine)
   {
      useRichText = true;
      text ="<b>Transfer small scales from linear RGB to " + hdrwtView.id +"</b>";
      toolTip = "<p>recovery of small structures after HDR dynamic compression" +
            "</p>";
   }

   // my ©

   this.lblCopyright = new Label(this)
   this.lblCopyright.text      = "© 2016, Hartmut V. Bornemann";

   // execute button

   this.execButton = new PushButton( this );
   with ( this.execButton ) {
      text = "Execute Transfer";
      enabled = false;
      onPress = function()
      {
         // define temporary views

         var hdrLargeScale;
         var hdrLuminance;

         var rawLargeScale;
         var rawLuminance;

         var LHEMask;

         enabled = false;

         dlg.cursor = new Cursor(StdCursor_Wait);

         Console.writeln("HDR: "+ hdrwtView.id);

         // make a copy of the raw image as this must be stretched

         Console.writeln("RAW: "+ rawView.id);

         var raw = copyOf(rawView, "_cpy");

         // stretch raw image

         maskedStretch(raw, hdrwtView.image.median());

         if (Console.abortRequested)
         {
            raw.window.forceClose();
         }
         else
         {
            // create a differenceMask from luminance images

            hdrLuminance = luminanceOf(hdrwtView);

            rawLuminance = luminanceOf(raw);

            // calc differenceMask = HDRWT_L - MS_L

            var differenceMask = getDifferenceMask(hdrLuminance, rawLuminance)

            hdrLuminance.window.forceClose();
            rawLuminance.window.forceClose();

            processMask(differenceMask);

            // large scale images

            hdrLargeScale = copyOf(hdrwtView, "_LS");
            blurr(hdrLargeScale);

            rawLargeScale = copyOf(raw, "_LS");
            blurr(rawLargeScale);

            // mask the image before transfer

            hdrwtView.window.mask = differenceMask.window;
            hdrwtView.window.maskEnabled = true;
            hdrwtView.window.maskVisible = true;

            // transfer small scales

            var continueI = iterate(
                              hdrwtView,
                              raw,
                              hdrLargeScale,
                              rawLargeScale,
                              iterations);

            hdrwtView.window.removeMask();

            // clean up, delete temporary views

            raw.window.forceClose();
            hdrLargeScale.window.forceClose();
            rawLargeScale.window.forceClose();
            if (!export_mask) differenceMask.window.forceClose();

            if (continueI)
            {
               // exit
               Console.writeln("");
               Console.writeln("<b>***ready***</b>");
               Console.writeln("");
            }
         }

         dlg.cursor = new Cursor(StdCursor_Arrow);
         dlg.done(0);
      }
   }

  this.buttonSizer = new HorizontalSizer;
   with ( this.buttonSizer )
   {
      spacing = 4;
      add( this.execButton );
   }

   // ------------------------------------------------------------------------
   //
   // RAW view selection
   //
   // ------------------------------------------------------------------------

   this.rawLabel = new Label( this );
   with ( this.rawLabel ) {
      text = "RAW image:";
      minWidth = labelWidth1;
   }

   this.rawViews = new ViewList(this);
   with ( this.rawViews )
   {
      getMainViews();
      remove(hdrwtView);
      toolTip = "<p>The RAW image corresponds to the linear " +
         "color calibrated RGB.</p>";

      onViewSelected = function( view )
      {
         if (!view.image.isGrayscale)
         {
             rawView = view;
             dlg.execButton.enabled = true;
         }
         else
         {
             luminance = null;
             dlg.execButton.enabled = false;
         }
      }
   }

   this.lumImageSizer = new HorizontalSizer;
   with ( this.lumImageSizer )
   {
      spacing = 4;
      add( this.rawLabel );
      add( this.rawViews );
   }


   // ------------------------------------------------------------------------
   // iterator numeric control
   // ------------------------------------------------------------------------

   this.iter_Label = new Label( this );
   with (this.iter_Label)
   {
      minWidth = labelWidth1;
      text = "Iterations:";
      textAlignment = TextAlign_Right|TextAlign_VertCenter;
   }

   this.iter_SpinBox = new SpinBox( this );
   with (this.iter_SpinBox)
   {
      minValue = 1;
      maxValue = 10;
      value = iterations;
      toolTip = "<p>Subtraction of differences in the luminance between " +
            "HDR compressed and the linear image can be " +
            "repeated, to get the desired stellar profiles</p>";
      onValueUpdated = function( value )
      {
         iterations = value;
      }
   }

   this.numISizer = new HorizontalSizer(this);
   with ( this.numISizer )
   {
      spacing = 4;
      add( this.iter_Label );
      add( this.iter_SpinBox );
   }


   this.iter_Sizer = new HorizontalSizer;
   with (this.iter_Sizer)
   {
      spacing = 4;
      add( this.iter_Label );
      add( this.iter_SpinBox );
      addStretch();
   }

   this.parmsGroupBox = new GroupBox( this );
   with (this.parmsGroupBox)
   {
      sizer = new VerticalSizer;
      sizer.margin = 6;
      sizer.spacing = 4;
      sizer.add( this.lumImageSizer );
      sizer.add( this.numISizer );
      title = "Settings";
   }


   this.exportCheckBox = new CheckBox()
   with (this.exportCheckBox)
   {
      text = "Export difference mask";
      onCheck = function ( checked )
      {
         export_mask = checked;
      }
   }

   this.exportGroupBox = new GroupBox( this );
   with (this.exportGroupBox)
   {
      title = "Object disposition";
      sizer = new VerticalSizer;
      sizer.margin = 6;
      sizer.spacing = 4;
      sizer.add( this.exportCheckBox );
   }

   // ------------------------------------------------------------------------

   this.input_GroupBox = new GroupBox( this );
   with ( this.input_GroupBox )
   {
      title = "";
      sizer = new VerticalSizer;
   }
   with ( this.input_GroupBox.sizer )
   {
      margin = 6;
      spacing = 4;
      add( this.parmsGroupBox );
      add( this.exportGroupBox);
      add( this.buttonSizer );
   }

   // ------------------------------------------------------------------------
   // finally arrange all sizing elements and ajust the dialog frame
   // ------------------------------------------------------------------------

   this.sizer = new VerticalSizer(this);
   this.sizer.margin = 6;
   this.sizer.spacing = 6;
   this.sizer.add( this.lblHeadLine );
   this.sizer.add( this.lblCopyright );
   this.sizer.add( this.input_GroupBox );
   this.windowTitle = "SST " + VERSION;

   this.adjustToContents();
}

function iterate(hdrwtView, rawView, hdrLargeScale, rawLargeScale, iterations)
{
   for (var i = 0; i < iterations; i++)
   {
      if (Console.abortRequested)
      {
         return false;
         break;
      }
      Console.writeln("Iteration " + (i + 1));
      transfer(hdrwtView, rawView, hdrLargeScale, rawLargeScale);
   }
   return true;
}

function luminanceOf(view)
{
   var name = getNewName(view.id, "_L");
   var P = new ChannelExtraction;
   P.colorSpace = ChannelExtraction.prototype.CIELab;
   P.channels = [ // enabled, id
   [true, name],
   [false, ""],
   [false, ""]
   ];
   P.sampleFormat = ChannelExtraction.prototype.SameAsSource;
   P.executeOn( view );
   return View.viewById(name);
}

function copyOf(view, suffix)
{
   var img = view.image;
   var name = getNewName(view.id, suffix);
   var wtmp = new ImageWindow( img.width, img.height, img.numberOfChannels,
      img.bitsPerSample, img.sampleType == SampleType_Real, img.isColor,name);
   var v = wtmp.mainView;
   v.beginProcess( UndoFlag_NoSwapFile );
   v.image.assign( img );
   v.endProcess();
   return v;
}

function getDifferenceMask(HDR_L, MS_L)
{
   var name = getNewName("difference", "_Mask");
   var P = new PixelMath;
   P.expression = HDR_L.id +" - " + MS_L.id;
   P.expression1 = "";
   P.expression2 = "";
   P.expression3 = "";
   P.useSingleExpression = true;
   P.symbols = "";
   P.generateOutput = true;
   P.singleThreaded = false;
   P.use64BitWorkingImage = false;
   P.rescale = false;
   P.rescaleLower = 0;
   P.rescaleUpper = 1;
   P.truncate = true;
   P.truncateLower = 0;
   P.truncateUpper = 1;
   P.createNewImage = true;
   P.showNewImage = true;
   P.newImageId = name;
   P.newImageWidth = HDR_L.image.width;
   P.newImageHeight = HDR_L.image.height;
   P.newImageAlpha = false;
   P.newImageColorSpace = PixelMath.prototype.Gray;
   P.newImageSampleFormat = PixelMath.prototype.i32;
   /*
    * Read-only properties
    *
   P.outputData = [ // globalVariableId, globalVariableRK, globalVariableG, globalVariableB
   ];
    */
   P.executeGlobal();
   return View.viewById(name);
}

function processMask(mask)
{
   var P = new MorphologicalTransformation;
   P.operator = MorphologicalTransformation.prototype.Dilation;
   P.interlacingDistance = 1;
   P.lowThreshold = 0.000000;
   P.highThreshold = 0.000000;
   P.numberOfIterations = 1;
   P.amount = 1.00;
   P.selectionPoint = 0.50;
   P.structureName = "";
   P.structureSize = 3;
   P.structureWayTable = [ // mask
      [[
         0x00,0x01,0x00,
         0x01,0x01,0x01,
         0x00,0x01,0x00
      ]]
   ];
   P.executeOn(mask);

   P = new ATrousWaveletTransform;
   P.layers = [ // enabled, biasEnabled, bias, noiseReductionEnabled, noiseReductionThreshold, noiseReductionAmount, noiseReductionIterations
      [false, true, 0.000, false, 3.000, 1.00, 1],
      [true, true, 0.000, false, 3.000, 1.00, 1],
      [true, true, 0.000, false, 3.000, 1.00, 1],
      [true, true, 0.000, false, 3.000, 1.00, 1],
      [true, true, 0.000, false, 3.000, 1.00, 1]
   ];
   P.scaleDelta = 0;
   P.scalingFunctionData = [
      0.111111,0.333333,0.111111,
      0.333333,1,0.333333,
      0.111111,0.333333,0.111111
   ];
   P.scalingFunctionRowFilter = [
      0.333333,
      1,
      0.333333
   ];
   P.scalingFunctionColFilter = [
      0.333333,
      1,
      0.333333
   ];
   P.scalingFunctionNoiseSigma = [
      0.6881,0.2865,0.1352,
      0.0672,0.034,0.0173,
      0.0089,0.0046,0.0024,
      0.0012
   ];
   P.scalingFunctionName = "Small Scale 3 (3)";
   P.largeScaleFunction = ATrousWaveletTransform.prototype.NoFunction;
   P.curveBreakPoint = 0.75;
   P.noiseThresholding = false;
   P.noiseThresholdingAmount = 1.00;
   P.noiseThreshold = 3.00;
   P.softThresholding = true;
   P.useMultiresolutionSupport = false;
   P.deringing = false;
   P.deringingDark = 0.1000;
   P.deringingBright = 0.0000;
   P.outputDeringingMaps = false;
   P.lowRange = 0.0000;
   P.highRange = 0.0000;
   P.previewMode = ATrousWaveletTransform.prototype.Disabled;
   P.previewLayer = 0;
   P.toLuminance = true;
   P.toChrominance = true;
   P.linear = false;
   P.executeOn(mask);
}

function processLHEMask(mask)
{
   // move Highlights near 0.22

   var P = new HistogramTransformation;
   P.H = [ // c0, m, c1, r0, r1
      [0.00000000, 0.50000000, 1.00000000, 0.00000000, 1.00000000],
      [0.00000000, 0.48325673, 1.00000000, 0.00000000, 1.00000000],
      [0.00000000, 0.50492449, 1.00000000, 0.00000000, 1.00000000],
      [0.00000000, 0.50000000, 0.22531646, 0.00000000, 1.00000000],
      [0.00000000, 0.50000000, 1.00000000, 0.00000000, 1.00000000]
   ];
   P.executeOn(mask);

   // first morphological transformation

   var P = new MorphologicalTransformation;
   P.operator = MorphologicalTransformation.prototype.Dilation;
   P.interlacingDistance = 1;
   P.lowThreshold = 0.000000;
   P.highThreshold = 0.000000;
   P.numberOfIterations = 1;
   P.amount = 1.00;
   P.selectionPoint = 0.50;
   P.structureName = "";
   P.structureSize = 5;
   P.structureWayTable = [ // mask
      [[
         0x00,0x00,0x01,0x00,0x00,
         0x00,0x01,0x01,0x01,0x00,
         0x01,0x01,0x01,0x01,0x01,
         0x00,0x01,0x01,0x01,0x00,
         0x00,0x00,0x01,0x00,0x00
      ]]
   ];
   P.executeOn(mask);

   // first smoothing with ATWT
   // Layers 1,2,3 disabled in a 4 layer structure

   P = new ATrousWaveletTransform;
   P.layers = [ // enabled, biasEnabled, bias, noiseReductionEnabled, noiseReductionThreshold, noiseReductionAmount, noiseReductionIterations
   [false, true, 0.000, false, 3.000, 1.00, 1],
   [false, true, 0.000, false, 3.000, 1.00, 1],
   [false, true, 0.000, false, 3.000, 1.00, 1],
   [true, true, 0.000, false, 3.000, 1.00, 1],
   [true, true, 0.000, false, 3.000, 1.00, 1]
   ];
   P.scaleDelta = 0;
   P.scalingFunctionData = [
   0.111111,0.333333,0.111111,
   0.333333,1,0.333333,
   0.111111,0.333333,0.111111
   ];
   P.scalingFunctionRowFilter = [
   0.333333,
   1,
   0.333333
   ];
   P.scalingFunctionColFilter = [
   0.333333,
   1,
   0.333333
   ];
   P.scalingFunctionNoiseSigma = [
   0.6881,0.2865,0.1352,
   0.0672,0.034,0.0173,
   0.0089,0.0046,0.0024,
   0.0012
   ];
   P.scalingFunctionName = "Small Scale 3 (3)";
   P.largeScaleFunction = ATrousWaveletTransform.prototype.NoFunction;
   P.curveBreakPoint = 0.75;
   P.noiseThresholding = false;
   P.noiseThresholdingAmount = 1.00;
   P.noiseThreshold = 3.00;
   P.softThresholding = true;
   P.useMultiresolutionSupport = false;
   P.deringing = false;
   P.deringingDark = 0.1000;
   P.deringingBright = 0.0000;
   P.outputDeringingMaps = false;
   P.lowRange = 0.0000;
   P.highRange = 0.0000;
   P.previewMode = ATrousWaveletTransform.prototype.Disabled;
   P.previewLayer = 0;
   P.toLuminance = true;
   P.toChrominance = true;
   P.linear = false;
   P.executeOn(mask);

   // second morphological transformation
   // Structuring element 5x5 zeroes (white)

   P = new MorphologicalTransformation;
   P.operator = MorphologicalTransformation.prototype.Dilation;
   P.interlacingDistance = 1;
   P.lowThreshold = 0.000000;
   P.highThreshold = 0.000000;
   P.numberOfIterations = 1;
   P.amount = 1.00;
   P.selectionPoint = 0.50;
   P.structureName = "";
   P.structureSize = 5;
   P.structureWayTable = [ // mask
      [[
         0x01,0x01,0x01,0x01,0x01,
         0x01,0x01,0x01,0x01,0x01,
         0x01,0x01,0x01,0x01,0x01,
         0x01,0x01,0x01,0x01,0x01,
         0x01,0x01,0x01,0x01,0x01
      ]]
   ];
   P.executeOn(mask);

   // second smoothing with ATWT

   P = new ATrousWaveletTransform;
   P.layers = [ // enabled, biasEnabled, bias, noiseReductionEnabled, noiseReductionThreshold, noiseReductionAmount, noiseReductionIterations
   [false, true, 0.000, false, 3.000, 1.00, 1],
   [false, true, 0.000, false, 3.000, 1.00, 1],
   [false, true, 0.000, false, 3.000, 1.00, 1],
   [true, true, 0.000, false, 3.000, 1.00, 1],
   [true, true, 0.000, false, 3.000, 1.00, 1]
   ];
   P.scaleDelta = 0;
   P.scalingFunctionData = [
   0.111111,0.333333,0.111111,
   0.333333,1,0.333333,
   0.111111,0.333333,0.111111
   ];
   P.scalingFunctionRowFilter = [
   0.333333,
   1,
   0.333333
   ];
   P.scalingFunctionColFilter = [
   0.333333,
   1,
   0.333333
   ];
   P.scalingFunctionNoiseSigma = [
   0.6881,0.2865,0.1352,
   0.0672,0.034,0.0173,
   0.0089,0.0046,0.0024,
   0.0012
   ];
   P.scalingFunctionName = "Small Scale 3 (3)";
   P.largeScaleFunction = ATrousWaveletTransform.prototype.NoFunction;
   P.curveBreakPoint = 0.75;
   P.noiseThresholding = false;
   P.noiseThresholdingAmount = 1.00;
   P.noiseThreshold = 3.00;
   P.softThresholding = true;
   P.useMultiresolutionSupport = false;
   P.deringing = false;
   P.deringingDark = 0.1000;
   P.deringingBright = 0.0000;
   P.outputDeringingMaps = false;
   P.lowRange = 0.0000;
   P.highRange = 0.0000;
   P.previewMode = ATrousWaveletTransform.prototype.Disabled;
   P.previewLayer = 0;
   P.toLuminance = true;
   P.toChrominance = true;
   P.linear = false;
   P.executeOn(mask);

   // Curves, hard contrast

   P = new CurvesTransformation;
   P.R = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.Rt = CurvesTransformation.prototype.AkimaSubsplines;
   P.G = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.Gt = CurvesTransformation.prototype.AkimaSubsplines;
   P.B = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.Bt = CurvesTransformation.prototype.AkimaSubsplines;
   P.K = [ // x, y
      [0.00000, 0.00000],
      [0.18088, 0.18088],
      [0.58915, 0.82687],
      [0.83204, 0.96899],
      [1.00000, 1.00000]
   ];
   P.Kt = CurvesTransformation.prototype.AkimaSubsplines;
   P.A = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.At = CurvesTransformation.prototype.AkimaSubsplines;
   P.L = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.Lt = CurvesTransformation.prototype.AkimaSubsplines;
   P.a = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.at = CurvesTransformation.prototype.AkimaSubsplines;
   P.b = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.bt = CurvesTransformation.prototype.AkimaSubsplines;
   P.c = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.ct = CurvesTransformation.prototype.AkimaSubsplines;
   P.H = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
   P.Ht = CurvesTransformation.prototype.AkimaSubsplines;
   P.S = [ // x, y
      [0.00000, 0.00000],
      [1.00000, 1.00000]
   ];
P.St = CurvesTransformation.prototype.AkimaSubsplines;

      P.executeOn(mask);
}

function localHistogram(view, contrast, amount)
{
   var P = new LocalHistogramEqualization;
   P.radius = 64;
   P.histogramBins = LocalHistogramEqualization.prototype.Bit8;
   P.slopeLimit = contrast;
   P.amount = amount;
   P.circularKernel = true;
   P.executeOn(view);
}

function transfer(HDRWT, MS, HDRWT_LS, MS_LS)
{
   /* transfer the small scales by
      MS - MS_LS + HDRWT_LS
      to HDRWT
   */
   var P = new PixelMath;
   P.expression =  MS.id + " - " + MS_LS.id + "+" + HDRWT_LS.id;
   P.expression1 = "";
   P.expression2 = "";
   P.expression3 = "";
   P.useSingleExpression = true;
   P.symbols = "";
   P.generateOutput = true;
   P.singleThreaded = false;
   P.use64BitWorkingImage = false;
   P.rescale = false;
   P.rescaleLower = 0;
   P.rescaleUpper = 1;
   P.truncate = true;
   P.truncateLower = 0;
   P.truncateUpper = 1;
   P.createNewImage = false;
   P.showNewImage = true;
   P.newImageId = "";
   P.newImageWidth = 0;
   P.newImageHeight = 0;
   P.newImageAlpha = false;
   P.newImageColorSpace = PixelMath.prototype.Gray;
   P.newImageSampleFormat = PixelMath.prototype.SameAsTarget;
   /*
    * Read-only properties
    *
   P.outputData = [ // globalVariableId, globalVariableRK, globalVariableG, globalVariableB
   ];
   */
   P.executeOn(HDRWT);
}

function maskedStretch(view, targetBackground)
{
   Console.writeln("Stretch target    "+ targetBackground);
   var P = new MaskedStretch;
   P.targetBackground = targetBackground;
   P.numberOfIterations = 100;
   P.clippingFraction = 0.00050000;
   P.backgroundReferenceViewId = "";
   P.backgroundLow = 0.00000000;
   P.backgroundHigh = 0.05000000;
   P.useROI = false;
   P.roiX0 = 0;
   P.roiY0 = 0;
   P.roiX1 = 0;
   P.roiY1 = 0;
   P.maskType = MaskedStretch.prototype.MaskType_Intensity;
   P.executeOn(view);
}

// Helper function
function getNewName(name, suffix)
{
   name += suffix;
   let n = 1;
   while (!ImageWindow.windowById(name).isNull)
   {  // if there are 99 windows with matching names then throw!!!
      ++n;
      name += n;
   }
   return name;
}

function blurr(view)
{
   var P = new ATrousWaveletTransform;
   P.layers = [ // enabled, biasEnabled, bias, noiseReductionEnabled, noiseReductionThreshold, noiseReductionAmount, noiseReductionIterations
      [false, true, 0.000, false, 3.000, 1.00, 1],
      [false, true, 0.000, false, 3.000, 1.00, 1],
      [false, true, 0.000, false, 3.000, 1.00, 1],
      [false, true, 0.000, false, 3.000, 1.00, 1],
      [false, true, 0.000, false, 3.000, 1.00, 1],
      [true, true, 0.000, false, 3.000, 1.00, 1]
   ];
   P.scaleDelta = 0;
   P.scalingFunctionData = [
      0.25,0.5,0.25,
      0.5,1,0.5,
      0.25,0.5,0.25
   ];
   P.scalingFunctionRowFilter = [
      0.5,
      1,
      0.5
   ];
   P.scalingFunctionColFilter = [
      0.5,
      1,
      0.5
   ];
   P.scalingFunctionNoiseSigma = [
      0.8003,0.2729,0.1198,
      0.0578,0.0287,0.0143,
      0.0072,0.0036,0.0019,
      0.001
   ];
   P.scalingFunctionName = "Linear Interpolation (3)";
   P.largeScaleFunction = ATrousWaveletTransform.prototype.NoFunction;
   P.curveBreakPoint = 0.75;
   P.noiseThresholding = false;
   P.noiseThresholdingAmount = 1.00;
   P.noiseThreshold = 3.00;
   P.softThresholding = true;
   P.useMultiresolutionSupport = false;
   P.deringing = false;
   P.deringingDark = 0.1000;
   P.deringingBright = 0.0000;
   P.outputDeringingMaps = false;
   P.lowRange = 0.0000;
   P.highRange = 0.0000;
   P.previewMode = ATrousWaveletTransform.prototype.Disabled;
   P.previewLayer = 0;
   P.toLuminance = true;
   P.toChrominance = true;
   P.linear = false;
   P.executeOn( view );
}

function message(txt)
{
   var bx = new MessageBox(txt, "Error");
   bx.execute();
}

sstDialog.prototype = new Dialog;

//////////////////////////////////////////////////////////////////////////////
//
// Main script entry point
//
function main()
{
      // Get access to the current active image window.
   var window = ImageWindow.activeWindow;
   if ( window.isNull )
   {
      message( "No active image" );
   }
   else
   {
      var img = window.currentView.image;
      if (!img.isColor)
      {
         message( "This script for RGB images only" );
      }
      else
      {
         Console.abortEnabled = true;
         var dialog = new sstDialog(window.currentView);
         dialog.execute();
      }
   }
}

main();
